% Correction application Chapitre 10
% Macroéconomie quantitative (2014, éd. Ellipses) 
% par Christophe Cahn

% Ce fichier permet de reproduire les élements contenu
% dans le fichier App_10_HA.pdf disponible sur le site
% www.christophecahn.fr/macroquant

% Filename : App_10_HA.m
% Application : Matlab/Octave
% Date : 24-04-2014
% Christophe Cahn (c) 2014

addpath('../MMMtbx');

rho = 0.9 ;
sig = 0.1 ;
nY  = 3   ;

[P,y,Pst]= markovappr(rho,sig,3,nY);
% P   : matrice stochastique
% y   : vecteur d'etats
% Pst : distribution invariante de la chaîne de Markov

% On en profite pour calculer le travail efficace agrégé
H = exp(y)*Pst ;

% Déclaration de l'objet model
model.params.gam 	  = 3    ;
model.params.bet 	  = 0.95 ;
model.params.alp 	  = 0.36 ;
model.params.dep 	  = 0.07 ;
model.params.sig 	  = sig  ;
model.params.rho 	  = rho  ;
model.params.Zed 	  = 1    ;
model.params.nStates  = nY   ;
model.params.phi 	  = 0   ;
model.params.assetMax = 30   ;
model.params.nAssets  = 200 ;
model.params.mPr 	  = P ;
model.params.vStates  = exp(y) ;
model.params.lab	  = H ;

model.options.tolOnDistribution = 1e-6 ;

% grille pour le taux d'intérêt
rmin = 0.033 ;
rmax = 0.051 ;
nr   = 10 ;
rate = linspace(rmin,rmax,nr)' ;

capitalSupply = zeros(nr,1) ;
capitalDemand = zeros(nr,1) ;

% et on itére
for ii=1:nr
	ii
	capitalSupply(ii) = computeCapitalSupply(rate(ii),model) ;
	capitalDemand(ii) = computeCapitalDemand(rate(ii),model) ;
end

figure;
plot(rate,capitalSupply,'bs',rate,capitalDemand,'r')
% plot(capitalDemand,rate,'r')
legend('Offre','Demande') ;	

% Trouver le taux d’équilibre
disp('calcul du taux d''équilibre');
f = @(r) computeCapitalSupply(r,model) - computeCapitalDemand(r,model) ;
rsol = bisection(f,[rmin rmax]) ;
yaxis = get(gca,'YLim') ;
line([rsol rsol], [yaxis(1) computeCapitalDemand(rsol,model)]) ;

% simulations
T     = 300 ;
nI    = 1000 ;
dr    = computeDecisionRule(rsol,model) ;
psi   = computeInvariantDistribution(dr,model) ;
amin  = model.params.phi ;
amax  = model.params.assetMax ;
na    = model.params.nAssets ;
grida = linspace(amin,amax,na) ;
r     = rsol ;
alp   = model.params.alp ;
Zed   = model.params.Zed ;
dep   = model.params.dep ;
w     = (1-alp)*(Zed*(alp/(r+dep))^alp)^(1/(1-alp));
% allocation de la memoire pour accelerer le code
a = zeros(nI,T) ;
c = zeros(nI,T) ;
s = zeros(nI,T) ;
for jj=1:nI
% simulation pour un menage
% On commence par trouver un etat initial tire
% dans la distribution stationnaire des chocs
% de productivité
[tmp,s1] = min(rand(1) >= cumsum(Pst)) ;
[s_t,s_id] = markov(P,T,s1,exp(y));
s_id = [((1:nY)'==s1),s_id];
% meme chose pour la richesse initiale
[tmp,a0] = min(rand(1) >= cumsum(sum(psi,2))) ;
% simulation
s(jj,1) = exp(y(s1)) ;
s(jj,2:T) = s_t ;
a(jj,1) = a0 ;
for ii=1:T-1
    % on selectionne la regle de decision correspondante
    % au choc pour trouver l’indice de a’
a1 = dr(a(jj,ii),:)*s_id(:,ii) ;
a(jj,ii+1) = a1 ;
c(jj,ii) = (1+r)*grida(a(jj,ii)) + w*s(jj,ii) - grida(a(jj,ii+1));
end
a1 = dr(a(jj,T),:)*s_id(:,T) ;
c(jj,T) = (1+r)*grida(a(jj,end)) + w*s(jj,end) - grida(a1);
end


% calcul des agregats
C = mean(c(:,101:end));
A = mean(grida(a(:,101:end)));
Y = Zed*A.^alp*H^(1-alp) ;
savings = 1-C./Y ;

% histogramme
hist(grida(a(:,T)))
plot(grida,sum(psi,2))
print('./App_10_hist.pdf','-dpdf')
disp('quantile')
y = quantile(grida(a(:,end)),[.05 .5 .95])
% gini coefficient and lorenz curve
[N,X] = hist(grida(a(:,end)));
gini(X',N')
plot([0 cumsum(N)/nI],[0 cumsum(X.*N)/sum(grida(a(:,end)))])
axis([0 1 0 1])
line([0 1],[0 1])
print('./App_10_gini.pdf','-dpdf')


